@@ -5,6 +5,7 @@ mod logout;
 mod manifest_to_json;
 mod package;
 mod publish;
+mod search;
 mod update;
 
 pub use init::InitSubcommand;
@@ -14,10 +15,9 @@ pub use logout::LogoutSubcommand;
 pub use manifest_to_json::ManifestToJsonSubcommand;
 pub use package::PackageSubcommand;
 pub use publish::PublishSubcommand;
+pub use search::SearchSubcommand;
 pub use update::UpdateSubcommand;
 
-use std::path::PathBuf;
-
 use structopt::StructOpt;
 
 #[derive(Debug, StructOpt)]
@@ -38,6 +38,7 @@ impl Args {
             Subcommand::Login(subcommand) => subcommand.run(),
             Subcommand::Logout(subcommand) => subcommand.run(),
             Subcommand::Update(subcommand) => subcommand.run(),
+            Subcommand::Search(subcommand) => subcommand.run(),
             Subcommand::Package(subcommand) => subcommand.run(),
             Subcommand::Install(subcommand) => subcommand.run(self.global),
             Subcommand::ManifestToJson(subcommand) => subcommand.run(),
@@ -53,11 +54,11 @@ pub struct GlobalOptions {
     #[structopt(global = true, parse(from_occurrences), long = "verbose", short)]
     pub verbosity: u8,
 
-    /// Overrides the registry with a local registry. Usable only by tests.
+    /// Flag to indidate if we will be using a test registry. Usable only by tests.
     #[structopt(skip)]
-    pub test_registry: Option<PathBuf>,
+    pub test_registry: bool,
 
-    /// Allows tests to specify if the package index should be temporary (to prevent multiple use conflicts). Usable only by tests.
+    /// Specify if the package index should be temporary (to prevent multiple use conflicts). Usable only by tests.
     #[structopt(skip)]
     pub use_temp_index: bool,
 }
@@ -66,7 +67,7 @@ impl Default for GlobalOptions {
     fn default() -> Self {
         Self {
             verbosity: 0,
-            test_registry: None,
+            test_registry: false,
             use_temp_index: false,
         }
     }
@@ -80,6 +81,7 @@ pub enum Subcommand {
     Publish(PublishSubcommand),
     Login(LoginSubcommand),
     Logout(LogoutSubcommand),
+    Search(SearchSubcommand),
     Package(PackageSubcommand),
     ManifestToJson(ManifestToJsonSubcommand),
 }
