@@ -6,14 +6,11 @@ use std::{
 
 use anyhow::format_err;
 use fs_err as fs;
-use indoc::formatdoc;
+use indoc::{formatdoc, indoc};
 
 use crate::{
-    manifest::Realm,
-    package_contents::PackageContents,
-    package_id::PackageId,
-    package_source::{PackageSourceId, PackageSourceMap},
-    resolution::Resolve,
+    manifest::Realm, package_contents::PackageContents, package_id::PackageId,
+    package_source::PackageSourceMap, resolution::Resolve,
 };
 
 pub struct InstallationContext {
@@ -27,7 +24,7 @@ pub struct InstallationContext {
 
 impl InstallationContext {
     /// Create a new `InstallationContext` for the given path.
-    pub fn new(project_path: &Path) -> Self {
+    pub fn new(project_path: &Path, shared_path: Option<String>) -> Self {
         let shared_dir = project_path.join("Packages");
         let shared_index_dir = shared_dir.join("_Index");
         let server_dir = project_path.join("ServerPackages");
@@ -36,7 +33,7 @@ impl InstallationContext {
         Self {
             shared_dir,
             shared_index_dir,
-            shared_path: None,
+            shared_path,
             server_dir,
             server_index_dir,
             server_path: None,
@@ -69,8 +66,6 @@ impl InstallationContext {
         root_package_id: PackageId,
         resolved: &Resolve,
     ) -> anyhow::Result<()> {
-        let default_registry = sources.get(&PackageSourceId::DefaultRegistry).unwrap();
-
         for package_id in &resolved.activated {
             log::debug!("Installing {}...", package_id);
 
@@ -81,11 +76,12 @@ impl InstallationContext {
             // package links for its dependencies.
             if package_id == &root_package_id {
                 if let Some(deps) = shared_deps {
-                    self.write_root_package_links(Realm::Shared, deps)?;
+                    self.write_root_package_links(Realm::Shared, deps, Realm::Shared)?;
+                    self.write_root_package_links(Realm::Server, deps, Realm::Shared)?;
                 }
 
                 if let Some(deps) = server_deps {
-                    self.write_root_package_links(Realm::Server, deps)?;
+                    self.write_root_package_links(Realm::Server, deps, Realm::Server)?;
                 }
             } else {
                 let metadata = resolved.metadata.get(package_id).unwrap();
@@ -104,7 +100,10 @@ impl InstallationContext {
                     self.write_package_links(package_id, package_realm, deps, Realm::Server)?;
                 }
 
-                let contents = default_registry.download_package(package_id)?;
+                let source_registry = &resolved.metadata[package_id].source_registry;
+                let package_source = sources.get(&source_registry).unwrap();
+                let contents = package_source.download_package(package_id)?;
+
                 self.write_contents(package_id, &contents, package_realm)?;
             }
         }
@@ -135,10 +134,14 @@ impl InstallationContext {
     /// Contents of a link into the shared index from outside the shared index.
     fn link_shared_index(&self, id: &PackageId) -> anyhow::Result<String> {
         let shared_path = self.shared_path.as_ref().ok_or_else(|| {
-            format_err!(
-                "Cannot have server dependencies depend on \
-                 shared dependencies without shared_path set"
-            )
+            format_err!(indoc! {r#"
+                A Server dependency is depending on a shared dependency.
+                To link these packages correctly you must declare the shared
+                package location in your wally.toml. This typically looks like:
+
+                [place]
+                shared-packages = "game.ReplicatedStorage.Packages"
+            "#})
         })?;
 
         let contents = formatdoc! {r#"
@@ -178,12 +181,13 @@ impl InstallationContext {
 
     fn write_root_package_links<'a, K: Display>(
         &self,
-        realm: Realm,
+        root_realm: Realm,
         dependencies: impl IntoIterator<Item = (K, &'a PackageId)>,
+        dependencies_realm: Realm,
     ) -> anyhow::Result<()> {
         log::debug!("Writing root package links");
 
-        let base_path = match realm {
+        let base_path = match root_realm {
             Realm::Shared => &self.shared_dir,
             Realm::Server => &self.server_dir,
         };
@@ -193,7 +197,12 @@ impl InstallationContext {
 
         for (dep_name, dep_package_id) in dependencies {
             let path = base_path.join(format!("{}.lua", dep_name));
-            let contents = self.link_root_same_index(dep_package_id);
+
+            let contents = match (root_realm, dependencies_realm) {
+                (source, dest) if source == dest => self.link_root_same_index(dep_package_id),
+                (_, Realm::Server) => self.link_server_index(dep_package_id)?,
+                (_, Realm::Shared) => self.link_shared_index(dep_package_id)?,
+            };
 
             log::trace!("Writing {}", path.display());
             fs::write(path, contents)?;
