@@ -1,15 +1,16 @@
-use std::io::{self, BufReader, Cursor};
-use std::path::Path;
+use std::io::{self, BufRead, BufReader, Cursor};
+use std::path::{Path, PathBuf};
 
 use anyhow::format_err;
 use fs_err::File;
+use globset::{Glob, GlobSet, GlobSetBuilder};
 use serde_json::json;
 use walkdir::WalkDir;
 use zip::{write::FileOptions, ZipArchive, ZipWriter};
 
 use crate::manifest::Manifest;
 
-static EXCLUDED_PATHS: &[&str] = &[".git", "wally.lock"];
+static EXCLUDED_GLOBS: &[&str] = &[".*", "wally.lock", "Packages", "ServerPackages"];
 
 /// Container for the contents of a package that have been downloaded.
 #[derive(Clone)]
@@ -26,15 +27,7 @@ impl PackageContents {
         let mut data = Vec::new();
         let mut archive = ZipWriter::new(Cursor::new(&mut data));
 
-        let file_iterator = WalkDir::new(input)
-            .min_depth(1)
-            .into_iter()
-            .filter_entry(|entry| dir_entry_filter(entry.path().strip_prefix(input).unwrap()));
-
-        for entry in file_iterator {
-            let entry = entry?;
-
-            let path = entry.path();
+        for path in Self::filtered_contents(input)? {
             let relative_path = path.strip_prefix(input).unwrap();
             let archive_name = relative_path.to_str().ok_or_else(|| {
                 format_err!(
@@ -43,7 +36,12 @@ impl PackageContents {
                 )
             })?;
 
-            if entry.file_type().is_dir() {
+            // Zips embed \ from windows paths causing incorrect extraction on unix operating
+            // systems; we must sanitise here. See: https://github.com/UpliftGames/wally/issues/15
+            // This may be fixed in the zip crate. See: https://github.com/zip-rs/zip/issues/253
+            let archive_name = str::replace(archive_name, "\\", "/");
+
+            if path.is_dir() {
                 archive.add_directory(archive_name, FileOptions::default())?;
             } else {
                 archive.start_file(archive_name, FileOptions::default())?;
@@ -89,6 +87,47 @@ impl PackageContents {
         Ok(())
     }
 
+    pub fn filtered_contents(input: &Path) -> anyhow::Result<Vec<PathBuf>> {
+        let manifest = Manifest::load(input)?;
+        let includes = manifest.package.include;
+        let mut excludes = manifest.package.exclude;
+
+        if includes.is_empty() && Path::new(".gitignore").exists() {
+            let gitignore = File::open(Path::new(".gitignore"))?;
+
+            BufReader::new(gitignore)
+                .lines()
+                .flatten()
+                .for_each(|pattern| {
+                    excludes.push(pattern);
+                });
+        }
+
+        EXCLUDED_GLOBS
+            .iter()
+            .map(|pattern| pattern.to_string())
+            .for_each(|pattern| excludes.push(pattern));
+
+        let include = build_glob_set(&includes)?;
+        let exclude = build_glob_set(&excludes)?;
+
+        Ok(WalkDir::new(input)
+            .min_depth(1)
+            .into_iter()
+            .filter_entry(|entry| {
+                let relative = entry.path().strip_prefix(input).unwrap();
+
+                if !includes.is_empty() && !include.matches(relative).is_empty() {
+                    return true;
+                };
+
+                exclude.matches(relative).is_empty()
+            })
+            .flatten()
+            .map(|entry| entry.path().to_path_buf())
+            .collect())
+    }
+
     pub fn data(&self) -> &[u8] {
         &self.data
     }
@@ -99,6 +138,12 @@ impl PackageContents {
     }
 }
 
-fn dir_entry_filter(path: &Path) -> bool {
-    !EXCLUDED_PATHS.iter().any(|p| Path::new(p) == path)
+fn build_glob_set(patterns: &[String]) -> anyhow::Result<GlobSet> {
+    let mut builder = GlobSetBuilder::new();
+
+    for pattern in patterns {
+        builder.add(Glob::new(pattern)?);
+    }
+
+    Ok(builder.build()?)
 }
