@@ -19,6 +19,9 @@ use crate::package_name::PackageName;
 pub struct PackageIndexConfig {
     pub api: Url,
     pub github_oauth_id: Option<String>,
+
+    #[serde(default)]
+    pub fallback_registries: Vec<String>,
 }
 
 pub struct PackageIndex {
@@ -69,6 +72,10 @@ impl PackageIndex {
         &self.url
     }
 
+    pub fn path(&self) -> &PathBuf {
+        &self.path
+    }
+
     pub fn new_temp(index_url: &Url, access_token: Option<String>) -> anyhow::Result<Self> {
         let temp_dir = tempfile::tempdir()?;
         let path = temp_dir.path().to_owned();
@@ -90,7 +97,10 @@ impl PackageIndex {
     pub fn update(&self) -> anyhow::Result<()> {
         let repository = self.repository.lock().unwrap();
 
-        log::info!("Updating package index...");
+        log::info!(
+            "Updating package index {}...",
+            repository.find_remote("origin")?.url().unwrap()
+        );
         git_util::update_index(self.access_token.clone(), &repository)
             .with_context(|| format!("could not update package index"))?;
 
@@ -171,9 +181,11 @@ impl PackageIndex {
                     .into_iter::<Manifest>()
                     .collect();
 
-            let versions = manifest_stream
+            let mut versions = manifest_stream
                 .with_context(|| format!("could not parse package index entry for {}", name))?;
 
+            versions.sort_by(|a, b| b.package.version.cmp(&a.package.version));
+
             let metadata = Arc::new(PackageMetadata { versions });
             package_cache.insert(name.clone(), Arc::clone(&metadata));
 
@@ -246,7 +258,7 @@ impl PackageIndex {
     }
 }
 
-#[derive(Default)]
+#[derive(Default, Serialize)]
 pub struct PackageMetadata {
     pub versions: Vec<Manifest>,
 }
