@@ -13,6 +13,10 @@ use crate::package_index::PackageIndex;
 use crate::package_req::PackageReq;
 use crate::package_source::{PackageContents, PackageSource};
 
+use super::PackageSourceId;
+
+const VERSION: &str = env!("CARGO_PKG_VERSION");
+
 pub struct Registry {
     index_url: Url,
     auth_token: OnceCell<Option<Arc<str>>>,
@@ -89,7 +93,7 @@ impl PackageSource for Registry {
 
         let url = self.api_url()?.join(&path)?;
 
-        let mut request = self.client.get(url);
+        let mut request = self.client.get(url).header("Wally-Version", VERSION);
 
         if let Some(token) = self.auth_token()? {
             request = request.header(AUTHORIZATION, format!("Bearer {}", token));
@@ -98,9 +102,11 @@ impl PackageSource for Registry {
 
         if !response.status().is_success() {
             bail!(
-                "Failed to download package {} from registry: {}",
+                "Failed to download package {} from registry: {}\n{} {}",
                 package_id,
-                response.status()
+                self.api_url()?,
+                response.status(),
+                response.text()?
             );
         }
 
@@ -109,4 +115,15 @@ impl PackageSource for Registry {
 
         Ok(PackageContents::from_buffer(data))
     }
+
+    fn fallback_sources(&self) -> anyhow::Result<Vec<PackageSourceId>> {
+        let fallback_registries = self.index()?.config()?.fallback_registries;
+
+        let sources = fallback_registries
+            .into_iter()
+            .map(PackageSourceId::Git)
+            .collect();
+
+        Ok(sources)
+    }
 }
