@@ -1,8 +1,7 @@
 import React from "react"
 import { Helmet } from "react-helmet"
-import { ButtonLink } from "../components/Button"
 import ContentSection from "../components/ContentSection"
-import { Heading } from "../components/Typography"
+import NotFoundMessage from "../components/NotFoundMessage"
 
 export default function NotFound() {
   return (
@@ -11,10 +10,7 @@ export default function NotFound() {
         <title>404: Not Found</title>
       </Helmet>
       <ContentSection variation="light">
-        <Heading>Lost Your Way?</Heading>
-
-        <ButtonLink to="/">Return Home</ButtonLink>
-        <p>HTTP 404: Page Not Found</p>
+        <NotFoundMessage errorMessage="HTTP 404: Page Not Found" />
       </ContentSection>
     </>
   )
